/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2004 by David Gerber <zapek@morphos.net>
 * Copyright 2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: appwindow.c,v 1.6 2006/04/12 17:51:47 fab Exp $
 */

#include "globals.h"

/* public */
#include <exec/memory.h>
#include <dos/dosextens.h>
#include <proto/exec.h>

/* private */
#include "clib/wb_protos.h"
#include "../ipc.h"
#include "qport.h"


#define DB_ADDAPPWINDOW 0
#define DB_REMOVEAPPWINDOW 0

struct appwindowmsg {
	struct ipcmessage    gmsg;
	struct ipc_appwindow msg;
};


struct AppWindow *AddAppWindowA(ULONG id, ULONG userdata, struct Window *window, struct MsgPort *msgport, struct TagItem *taglist)
{
	if (window && msgport)
	{
		struct appwindowmsg *msg;

		if ( (msg = AllocMem(sizeof(*msg), MEMF_PUBLIC)) )
		{
			struct Task *me = FindTask(NULL); /* we must not wait on ourself (eg, MUI config window is an appwindow as well) */
			struct MsgPort *amp;
			struct MsgPort rport;

			CreateQPort(&rport);

			msg->gmsg.pool             = NULL;
			msg->gmsg.msg.mn_ReplyPort = &rport;
			msg->gmsg.msg.mn_Length    = sizeof(msg->gmsg);
			msg->gmsg.type             = IPC_ADDAPPWINDOW;
			msg->gmsg.msgtype          = &msg->msg;

			msg->msg.id                = id;
			msg->msg.userdata          = userdata;
			msg->msg.userport          = msgport;
			msg->msg.window            = window;

			Forbid();
			if ( (amp = FindPort("Ambient IPC")) )
			{
				if (me != amp->mp_SigTask)
				{
					PutMsg(amp, &msg->gmsg.msg);
				}
			}
			Permit();

			if (amp && (me != amp->mp_SigTask))
			{
				WaitPort(&rport);
				(void)GetMsg(&rport);
				DeleteQPort(&rport);
				return ((struct AppWindow *)msg);
			}
			DeleteQPort(&rport);
		}
	}
	return (0);
}


BOOL RemoveAppWindow(struct AppWindow *appWindow)
{
	if (appWindow)
	{
		struct MsgPort *amp;
		struct appwindowmsg *msg = (struct appwindowmsg *)appWindow;
		struct MsgPort rport;

		CreateQPort(&rport);

		D(REMOVEAPPWINDOW,bug("msgport created.. reusing message\n"));

		msg->gmsg.type             = IPC_REMAPPWINDOW; /* reuse the message */
		msg->gmsg.msg.mn_ReplyPort = &rport;

		Forbid();
		if ( (amp = FindPort("Ambient IPC")) )
		{
			PutMsg(amp, &msg->gmsg.msg);
		}
		Permit();

		if (amp)
		{
			D(REMOVEAPPWINDOW,bug("msg sent.. waiting for reply..\n"));
			WaitPort(&rport);
			(void)GetMsg(&rport);
		}
		else
		{
			/*
			 * Ouch.. Ambient went away.. We
			 * cleanup ourself anyway.
			 */
		}
		D(REMOVEAPPWINDOW,bug("cleaning up..\n"));
		FreeMem(msg, sizeof(*msg));
		DeleteQPort(&rport);
	}
	return (0);
}
